/*
 * cksumtest.c
 *
 * Module test for in_cksum()
 *
 * Copyright 2008 Jaap Keuter and Greg Neujahr
 * Released into public domain.
 *
 * This module test goes through several specifically crafted data sets
 * in order to see what result the different checksum functions yield
 * with these sets.
 * The data sets consist of even and odd length sets starting at even
 * and add addresses. This in order to exercise all in_cksum optimizations.
 */

#include <glib.h>
#include "in_cksum.h"

/* A proposed checksum routine giving real live CDP checksums */
unsigned short real_cdp_checksum(const char *data, int length) {
        unsigned long sum = 0;
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        unsigned short value;

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                value = htons(value);
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

/* Wiresharks' CDP checksum routine verification, based on in_cksum() */
unsigned short wireshark_cdp_checksum(const char *data, int length) {
        unsigned long sum = 0;
        const unsigned short *d = (const unsigned short *)data;
        unsigned char *ptr;
        unsigned short value;

        while (length > 1) {
                sum += *d++;
                length -= 2;
        }
        if (length) {
                ptr = (unsigned char *)d;
                value = *ptr;
                sum += value;
        }

        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        sum = ~sum;
        sum = (unsigned short)sum;
        return sum;
}

int main(int argc, char *argv[])
{
  int checksum, i, j;
  vec_t vector[1];
  const char dataset1[] __attribute__ ((aligned)) = {0xfe, 0x00};
  const char dataset2[] __attribute__ ((aligned)) = {0x00, 0xfe};
  const char dataset3[] __attribute__ ((aligned)) = {0x00, 0xfe, 0x00};
  const char dataset4[] __attribute__ ((aligned)) = {0x00, 0x00, 0xfe};
  const char dataset5[] __attribute__ ((aligned)) = {0x00, 0x00, 0xfe, 0x00};
  const char dataset6[] __attribute__ ((aligned)) = {0x00, 0x00, 0x00, 0xfe};

  const int testcases = 12;

  for (i=0; i<testcases; i++)
  {
    switch (i)
    {
    case 0:
      vector[0].ptr = dataset1;
      vector[0].len = sizeof(dataset1);
      break;
    case 1:
      vector[0].ptr = dataset2;
      vector[0].len = sizeof(dataset2);
      break;
    case 2:
      vector[0].ptr = dataset3;
      vector[0].len = sizeof(dataset3);
      break;
    case 3:
      vector[0].ptr = dataset4;
      vector[0].len = sizeof(dataset4);
      break;
    case 4:
      vector[0].ptr = dataset5;
      vector[0].len = sizeof(dataset5);
      break;
    case 5:
      vector[0].ptr = dataset6;
      vector[0].len = sizeof(dataset6);
      break;
    case 6:
      vector[0].ptr = dataset1+1;
      vector[0].len = sizeof(dataset1)-1;
      break;
    case 7:
      vector[0].ptr = dataset2+1;
      vector[0].len = sizeof(dataset2)-1;
      break;
    case 8:
      vector[0].ptr = dataset3+1;
      vector[0].len = sizeof(dataset3)-1;
      break;
    case 9:
      vector[0].ptr = dataset4+1;
      vector[0].len = sizeof(dataset4)-1;
      break;
    case 10:
      vector[0].ptr = dataset5+1;
      vector[0].len = sizeof(dataset5)-1;
      break;
    case 11:
      vector[0].ptr = dataset6+1;
      vector[0].len = sizeof(dataset6)-1;
      break;

    default:
      g_print("\n ! INVALID TEST CASE ! \n");
      return;
    }

    g_print("\ntest %u data: %p length: %u\t", i, vector[0].ptr, vector[0].len);
    for (j=0; j<vector[0].len; j++)
      g_print("%02x ", *(vector[0].ptr+j));
    g_print("\n");

    checksum = in_cksum(vector, 1);

    g_print("in_cksum\t %d\n", checksum);

    checksum = wireshark_cdp_checksum(vector[0].ptr, vector[0].len);

    g_print("WS CDP checksum\t %d\n", checksum);

    checksum = real_cdp_checksum(vector[0].ptr, vector[0].len);

    g_print("CDP checksum\t %d\n", checksum);
  }
}
